/****************************************************************************

  Drago 3D
  Interactive animated 3D version of the Curlicue Fractal
  by Fergus Murray, 30/11/2002.

****************************************************************************/

#include <string.h>
#include <GL/glut.h>
#include "glui.h"
#include <math.h>

float xy_aspect;
int   last_x, last_y;
float rotationX = 0.0, rotationY = 0.0;

/** These are the live variables passed into GLUI ***/
int   wireframe = 1, paused=0, direction=1, reverse=0, mirror=1, trails=0;
int   obj_type = 1;
int   segments = 8;
int   segments2 = 8;
char  text[sizeof(GLUI_String)] = {" "};
/*int   light0_enabled = 1;
int   light1_enabled = 0;
float light0_intensity = 1.0;
float light1_intensity = 1.0;*/
int   main_window;
int   counter = 0;
float scale = 1.0;
float offset = 0;
float shimscale = 0.01f, dragscale=0.01f, piecesize=0.0025;
float angle, distance, colour;
float xcentre=39.9f, ycentre=40.1f;
int size=80, i=0, length=1000;
float time = 0.0f, input=0.11, recip=9.0909090909;

// Dragoric variables
float x, y, z, zscale=0.0002, speed=0.3;
double f, d, rate, seed;
double pi=3.14159265358979323846264338327950288419716939937510582097494459230781640628620899;
/** Pointers to the windows and some of the controls we'll create **/
GLUI *cmd_line_glui, *glui;
GLUI_Checkbox   *checkbox, *pausebox, *reversebox, *mirrorbox, *trailbox;
GLUI_Spinner    *spinner, *scale_spinner, *zscale_spinner, *speed_spinner, *piecesize_spinner;//, *light0_spinner, *light1_spinner
GLUI_RadioGroup *radio;
GLUI_EditText   *seedtext, *reciptext, *cmd_line;
GLUI_Panel      *obj_panel;

/********** User IDs for callbacks ********/
#define CMD_LINE_ID          100
/*#define LIGHT0_ENABLED_ID    200
#define LIGHT1_ENABLED_ID    201
#define LIGHT0_INTENSITY_ID  250
#define LIGHT1_INTENSITY_ID  251*/
#define INPUTRECIP_ID		 252
#define INPUTSEED_ID		 253
#define REVERSE_ID			 254

/********** Miscellaneous global variables **********/

GLfloat light0_ambient[] =  {0.1f, 0.1f, 0.3f, 1.0f};
GLfloat light0_diffuse[] =  {.6f, .6f, 1.0f, 1.0f};
GLfloat light0_position[] = {.5f, .5f, 1.0f, 0.0f};

GLfloat light1_ambient[] =  {0.1f, 0.1f, 0.3f, 1.0f};
GLfloat light1_diffuse[] =  {.9f, .6f, 0.0f, 1.0f};
GLfloat light1_position[] = {-1.0f, -1.0f, 1.0f, 0.0f};

/**************************************** control_cb() *******************/
/* GLUI control callback                                                 */

void control_cb( int control )
{
  /*
  Not used, code kept in case it becomes useful later:
  if ( control == CMD_LINE_ID ) {
	 // User typed text into the 'command line' window
	 printf( "Command (%d): %s\n", counter, cmd_line->get_text() );
	 cmd_line->set_text( "" );  /* Clear command line
  }
  else */
	if ( control == INPUTSEED_ID) { 
		seed=(double)input*pi*2;
		recip=1/input;
	}
	if ( control == INPUTRECIP_ID) {
		seed=(double)pi*2/recip;  
  		input=1/recip;
	}
  else if ( control == REVERSE_ID) direction=-direction;
}

/**************************************** myGlutKeyboard() **********/

void myGlutKeyboard(unsigned char Key, int x, int y)
{
  switch(Key)
  {
  case 27: 
  case 'q':
    exit(0);
    break;
  };
  
  glutPostRedisplay();
}


/***************************************** myGlutMenu() ***********/

void myGlutMenu( int value )
{
  myGlutKeyboard( value, 0, 0 );
}


/***************************************** myGlutIdle() ***********/

void myGlutIdle( void )
{
  /* According to the GLUT specification, the current window is 
     undefined during an idle callback.  So we need to explicitly change
     it if necessary */
  if ( glutGetWindow() != main_window ) 
    glutSetWindow(main_window);  


  glutPostRedisplay();

  /****************************************************************/
  /*            This demonstrates GLUI::sync_live()               */
  /*   We change the value of a variable that is 'live' to some   */
  /*   control.  We then call sync_live, and the control          */
  /*   associated with that variable is automatically updated     */
  /*   with the new value.  This frees the programmer from having */
  /*   to always remember which variables are used by controls -  */
  /*   simply change whatever variables are necessary, then sync  */
  /*   the live ones all at once with a single call to sync_live  */
  /****************************************************************/


  //time+=0.025f;
  
  glui->sync_live();

}


void curl(int backwards){
// Curlicue Fractal code begins

  x=0;
  y=0;
  if (mirror) z=0;
  else z=-length*zscale*0.5;
  f=0;
  d=0;
  
  if(!paused){
  counter++;
  rate=speed*speed*0.000001*direction;
  seed+=rate;
  }
  /*if (backwards==1) {
	d+=seed;
	f+=d;
	x+=cos(f)*dragscale*backwards;
	y+=sin(f)*dragscale*backwards;
	z+=zscale*backwards;
  }*/

  if (obj_type==0){
	  glBegin (GL_POINTS);
  }
  else if (obj_type==1){
	  if (wireframe)  glBegin (GL_LINE_STRIP);
	  else			glBegin (GL_TRIANGLE_STRIP);
  }
  for (i=0; i<length; i++){
	  glColor3d( 0.5*(1+cos(f*2)), 0.5*(1+sin(f*2)), 0.5*(1+sin(d))); 
	  if (obj_type==0||obj_type == 1) glVertex3f( x, y, z); 
	  else if ( obj_type == 2 ) {
		glPushMatrix();
		glTranslatef(x, y, z);
		if ( wireframe )      
		  glutWireCube( piecesize );
		else                  
		  glutSolidCube( piecesize );
		glPopMatrix();
	  }
	  else if ( obj_type == 3 ) {
		glPushMatrix();
		glTranslatef(x, y, z);
		if ( wireframe )      
		  glutWireSphere( piecesize , segments, segments );
		else                  
		  glutSolidSphere( piecesize , segments, segments );
		glPopMatrix();
	  }
	  f+=d;
	  d+=seed;
	  x+=cos(f)*dragscale*backwards;
	  y+=sin(f)*dragscale*backwards;
	  z+=zscale*backwards;
	  

  }
  if (obj_type==0||obj_type == 1) glEnd();
  // Curlicue code ends
}


/***************************************** myGlutMouse() **********/

void myGlutMouse(int button, int button_state, int x, int y )
{
  if ( button == GLUT_LEFT_BUTTON && button_state == GLUT_DOWN ) {
    last_x = x;
    last_y = y;
  }
}


/***************************************** myGlutMotion() **********/

void myGlutMotion(int x, int y )
{
  rotationX += (float) (y - last_y);
  rotationY += (float) (x - last_x);

  last_x = x;
  last_y = y;

  glutPostRedisplay(); 
}

/**************************************** myGlutReshape() *************/

void myGlutReshape( int x, int y )
{
  xy_aspect = (float)x / (float)y;
  glViewport( 0, 0, x, y );

  glutPostRedisplay();
}

/***************************************** myGlutDisplay() *****************/

void myGlutDisplay( void )
{
  glClearColor( .1f, .1f, .1f, 1.0f );
  if (!trails) glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();
  glFrustum( -xy_aspect*.08, xy_aspect*.08, -.08, .08, .1, 15.0 );

  glMatrixMode( GL_MODELVIEW );
  glLoadIdentity();
  glTranslatef( 0.0, 0.0, -1.2f );
  glRotatef( rotationX, 1.0, 0.0, 0.0 );
  glRotatef( rotationY, 0.0, 0.0, 1.0 );

  glScalef( scale, scale, scale );

  /*** Now we render object, using _live_ variables, which are transparently
	 updated by GLUI ***/

	curl(1);
	if (mirror) curl(-1);


  /* Disable lighting and set up ortho projection to render text*/
  
  glDisable( GL_LIGHTING );
  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();
  gluOrtho2D( 0.0, 100.0, 0.0, 100.0  );
  glMatrixMode( GL_MODELVIEW );
  glLoadIdentity();
  glColor3ub( 1, 1, 1 );
  glRasterPos2i( 10, 10 ); 

  /*** Render the live character array 'text'***/
  int i;
  for( i=0; i<(int)strlen( text ); i++ )
	 glutBitmapCharacter( GLUT_BITMAP_HELVETICA_18, text[i] );

  //glEnable( GL_LIGHTING );

  glutSwapBuffers();
}

/**************************************** main() ********************/

void main(int argc, char* argv[])
{
  /****************************************/
  /*   Initialize GLUT and create window  */
  /****************************************/

  glutInitDisplayMode( GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH );
  glutInitWindowPosition( 10, 10 );
  glutInitWindowSize( 600, 600 );

  main_window = glutCreateWindow( "Interactive 3D Curlicue Fractal" );
  glutDisplayFunc( myGlutDisplay );
  glutReshapeFunc( myGlutReshape );
  glutKeyboardFunc( myGlutKeyboard );
  glutMotionFunc( myGlutMotion );
  glutMouseFunc( myGlutMouse );

  /****************************************/
  /*       Set up OpenGL lights           */
  /****************************************/

  // Lighting currently disabled; not necessary for this kind of display.

  //glEnable(GL_LIGHTING);
  glEnable( GL_NORMALIZE );

  /*glEnable(GL_LIGHT0);
  glLightfv(GL_LIGHT0, GL_AMBIENT, light0_ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, light0_diffuse);
  glLightfv(GL_LIGHT0, GL_POSITION, light0_position);

  glLightfv(GL_LIGHT1, GL_AMBIENT, light1_ambient);
  glLightfv(GL_LIGHT1, GL_DIFFUSE, light1_diffuse);
  glLightfv(GL_LIGHT1, GL_POSITION, light1_position);
  */
  /****************************************/
  /*          Enable z-buferring          */
  /****************************************/

  glEnable(GL_DEPTH_TEST);

  /****************************************/
  /*         Here's the GLUI code         */
  /****************************************/

  printf( "GLUI version: %3.2f\n", GLUI_Master.get_version() );

  glui = GLUI_Master.create_glui( "GLUI", 0, 620, 50 ); /* name, flags,
								x, and y */
  glui->add_statictext( "GLUI Example 3" );
  obj_panel = glui->add_panel( "Object" );

  /***** Get seed value from input box ***/

  control_cb (INPUTSEED_ID);

  /***** Control for the object type *****/

  GLUI_Panel *type_panel = glui->add_panel_to_panel( obj_panel, "Type");
  radio = glui->add_radiogroup_to_panel(type_panel,&obj_type,4,control_cb);
  glui->add_radiobutton_to_group( radio, "Points" );
  glui->add_radiobutton_to_group( radio, "Line" );
  glui->add_radiobutton_to_group( radio, "Cubes" );
  glui->add_radiobutton_to_group( radio, "Spheres" );

  checkbox =
	 glui->add_checkbox_to_panel( obj_panel, "Wireframe", &wireframe, 1,
				 control_cb );
  trailbox =
	 glui->add_checkbox_to_panel( obj_panel, "Trails", &trails, 1,
				 control_cb );
  pausebox =
	 glui->add_checkbox_to_panel( obj_panel, "Paused", &paused, 1,
				 control_cb );
  reversebox =
	 glui->add_checkbox_to_panel( obj_panel, "Reverse direction", &reverse, REVERSE_ID,
				 control_cb );
  mirrorbox =
	 glui->add_checkbox_to_panel( obj_panel, "Double-ended", &mirror, REVERSE_ID,
				 control_cb );
  
  spinner  = glui->add_spinner_to_panel( obj_panel, "Length:",
					 GLUI_SPINNER_INT, &length);
  spinner->set_int_limits( 2, 10000 );
  spinner->set_alignment( GLUI_ALIGN_RIGHT );

  scale_spinner =
	 glui->add_spinner_to_panel( obj_panel, "Scale:",
				GLUI_SPINNER_FLOAT, &scale);
  scale_spinner->set_float_limits( .2f, 20.0 );
  scale_spinner->set_alignment( GLUI_ALIGN_RIGHT );

  zscale_spinner =
	 glui->add_spinner_to_panel( obj_panel, "Extrusion:",
				GLUI_SPINNER_FLOAT, &zscale);
  zscale_spinner->set_float_limits( 0, 0.005 );
  zscale_spinner->set_alignment( GLUI_ALIGN_RIGHT );

  piecesize_spinner =
	 glui->add_spinner_to_panel( obj_panel, "Piece Size:",
				GLUI_SPINNER_FLOAT, &piecesize);
  piecesize_spinner->set_float_limits( 0, 0.01 );
  piecesize_spinner->set_alignment( GLUI_ALIGN_RIGHT );

  speed_spinner =
	 glui->add_spinner_to_panel( obj_panel, "Speed:",
				GLUI_SPINNER_FLOAT, &speed);
  speed_spinner->set_float_limits( 0, 10.0 );
  speed_spinner->set_alignment( GLUI_ALIGN_RIGHT );

  glui->add_separator_to_panel( obj_panel );
  seedtext = glui->add_edittext_to_panel( obj_panel, "Seed:",
					  GLUI_EDITTEXT_FLOAT, &input, INPUTSEED_ID, control_cb );
  seedtext->set_w( 150 );

  reciptext = glui->add_edittext_to_panel( obj_panel, "Reciprocal of seed:",
					  GLUI_EDITTEXT_FLOAT, &recip, INPUTRECIP_ID, control_cb );
  reciptext->set_w( 150 );

  /******** Add some controls for lights

  GLUI_Panel *light0 = glui->add_panel( "Light 1" );
  GLUI_Panel *light1 = glui->add_panel( "Light 2" );

  glui->add_checkbox_to_panel( light0, "Enabled", &light0_enabled,
					 LIGHT0_ENABLED_ID, control_cb );
  light0_spinner =
	 glui->add_spinner_to_panel( light0, "Intensity:", GLUI_SPINNER_FLOAT,
				&light0_intensity, LIGHT0_INTENSITY_ID,
				control_cb );
  light0_spinner->set_float_limits( 0.0, 1.0 );

  glui->add_checkbox_to_panel( light1, "Enabled", &light1_enabled,
					 LIGHT1_ENABLED_ID, control_cb );
  light1_spinner =
	 glui->add_spinner_to_panel( light1, "Intensity:", GLUI_SPINNER_FLOAT,
				&light1_intensity, LIGHT1_INTENSITY_ID,
				control_cb );
  light1_spinner->set_float_limits( 0.0, 1.0 );
  light1_spinner->disable();   ********//* Disable this light initially */

  /****** Add a grayed-out counter *****/

  GLUI_EditText *counter_edittext =
	 glui->add_edittext( "Count:", GLUI_EDITTEXT_INT, &counter );
  counter_edittext->disable();

  /****** A 'quit' button *****/

  glui->add_button( "Quit", 0,(GLUI_Update_CB)exit );

  /****** Command line window ******/
/*
// Disused.

  cmd_line_glui = GLUI_Master.create_glui( "Enter command:",
						0, 50, 620 );

  cmd_line = cmd_line_glui->add_edittext( "Command:",
					  GLUI_EDITTEXT_TEXT, NULL,
					  CMD_LINE_ID, control_cb );
  cmd_line->set_w( 400 );  *//** Widen 'command line' control **/

  /**** Link windows to GLUI, and register idle callback ******/

  glui->set_main_gfx_window( main_window );
  //cmd_line_glui->set_main_gfx_window( main_window );

  /* We register the idle callback with GLUI, not with GLUT */
  GLUI_Master.set_glutIdleFunc( myGlutIdle );

  /**** Regular GLUT main loop ****/
  glutMainLoop();
}

